/********************************************************************************************************
 * @file    pbkdf2_test.c
 *
 * @brief   This is the source file for B91m
 *
 * @author  Driver Group
 * @date    2019
 *
 * @par     Copyright (c) 2019, Telink Semiconductor (Shanghai) Co., Ltd. ("TELINK")
 *
 *          Licensed under the Apache License, Version 2.0 (the "License");
 *          you may not use this file except in compliance with the License.
 *          You may obtain a copy of the License at
 *
 *              http://www.apache.org/licenses/LICENSE-2.0
 *
 *          Unless required by applicable law or agreed to in writing, software
 *          distributed under the License is distributed on an "AS IS" BASIS,
 *          WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *          See the License for the specific language governing permissions and
 *          limitations under the License.
 *
 *******************************************************************************************************/
#include <stdio.h>
#include "app_config.h"

#if(HASH_TEST_MODE == PBKDF2_TEST)



//#define PBKDF2_HMAC_SPEED_TEST_BY_TIMER

#ifdef PBKDF2_HMAC_SPEED_TEST_BY_TIMER
extern unsigned int startP(void);
extern unsigned int endP(unsigned char mode, unsigned int once_bytes, unsigned int round);
#endif



unsigned int pbkdf2_hmac_test_internal(HASH_ALG hash_alg, const unsigned char *pwd, unsigned short sp_key_idx, unsigned int pwd_bytes, const unsigned char *salt,
        unsigned int salt_bytes, unsigned int iter, const unsigned char *std_out, unsigned int std_out_bytes, const char *info)
{
    char *name[] = {"SM3","MD5","SHA256","SHA384","SHA512","SHA1","SHA224","SHA512_224","SHA512_256"};
    unsigned char out[1000];
    unsigned int ret;

    ret = pbkdf2_hmac(hash_alg, (unsigned char *)pwd, sp_key_idx, pwd_bytes, (unsigned char *)salt, salt_bytes, iter, out, std_out_bytes);
    if((HASH_SUCCESS != ret) || memcmp_(out, (unsigned char *)std_out, std_out_bytes))
    {
        printf("\r\n PBKDF2-HMAC-%s test %s failure, ret=%x\r\n", name[hash_alg], info, ret);
        print_buf_U8((unsigned char *)pwd, pwd_bytes, "password");
        print_buf_U8((unsigned char *)salt, salt_bytes, "salt");
        printf("\r\n iteration counter = %u \r\n", iter);
        print_buf_U8(out, std_out_bytes, "out");
        return 1;
    }
    else
    {
        printf("\r\n PBKDF2-HMAC-%s test %s success", name[hash_alg], info);fflush(stdout);
        return 0;
    }
}




const unsigned char password_1[8]     = {0x70,0x61,0x73,0x73,0x77,0x6F,0x72,0x64};//"password"
const unsigned char salt_1[4]         = {0x73,0x61,0x6C,0x74};                    //"salt"

const unsigned char password_2[24]    = {0x70,0x61,0x73,0x73,0x77,0x6F,0x72,0x64,0x50,0x41,0x53,0x53,0x57,0x4F,0x52,0x44,
                                   0x70,0x61,0x73,0x73,0x77,0x6F,0x72,0x64,};//"passwordPASSWORDpassword"
const unsigned char salt_2[36]        = {0x73,0x61,0x6C,0x74,0x53,0x41,0x4C,0x54,0x73,0x61,0x6C,0x74,0x53,0x41,0x4C,0x54,
                                   0x73,0x61,0x6C,0x74,0x53,0x41,0x4C,0x54,0x73,0x61,0x6C,0x74,0x53,0x41,0x4C,0x54,
                                   0x73,0x61,0x6C,0x74,};                    //"saltSALTsaltSALTsaltSALTsaltSALTsalt"

const unsigned char password_3[9]     = {0x70,0x61,0x73,0x73,0x00,0x77,0x6F,0x72,0x64};//"pass\0word"
const unsigned char salt_3[5]         = {0x73,0x61,0x00,0x6C,0x74};                    //"sa\0lt"

const unsigned char password_4[6]     = {0x70,0x61,0x73,0x73,0x77,0x64};//"passwd"

const unsigned char password_5[8]     = {0x50,0x61,0x73,0x73,0x77,0x6F,0x72,0x64};//"Password"
const unsigned char salt_5[4]         = {0x4E,0x61,0x43,0x6C};//"NaCl"


#ifdef SUPPORT_HASH_MD5
const unsigned char md5_dk_1[16]      = {0xF3,0x1A,0xFB,0x6D,0x93,0x13,0x92,0xDA,0xA5,0xE3,0x13,0x0F,0x47,0xF9,0xA9,0xB6};
const unsigned char md5_dk_2[16]      = {0x04,0x24,0x07,0xB5,0x52,0xBE,0x34,0x5A,0xD6,0xEE,0xE2,0xCF,0x2F,0x7E,0xD0,0x1D};
const unsigned char md5_dk_3[16]      = {0x15,0x00,0x1F,0x89,0xB9,0xC2,0x9E,0xE6,0x99,0x8C,0x52,0x0D,0x1A,0x06,0x29,0xE8};
const unsigned char md5_dk_4[16]      = {0xF9,0xAD,0xCC,0x1B,0x44,0xC0,0x72,0x85,0xB6,0xCD,0xCA,0xDF,0x75,0xEB,0xF7,0x2D};
const unsigned char md5_dk_5[25]      = {0x8D,0x5D,0x0A,0xAD,0x94,0xD1,0x44,0x20,0x42,0x9F,0xBC,0x7E,0x5B,0x08,0x7D,0x7A,
                                   0x55,0x27,0xE6,0x5D,0xFD,0x0D,0x48,0x6A,0x31};
const unsigned char md5_dk_6[16]      = {0x9D,0x35,0xF8,0xF9,0xB2,0x6A,0xA9,0xA8,0xAA,0x39,0x27,0x66,0x34,0xC6,0x79,0x3A,};
const unsigned char md5_dk_7[64]      = {0x91,0xD2,0xA2,0x0A,0x40,0x04,0x4E,0x9A,0x4D,0x74,0x12,0xEC,0x5E,0xF1,0xCC,0x08,
                                   0x07,0xE1,0x1F,0xB7,0xBB,0x9D,0x06,0x36,0xCB,0x6F,0x68,0x37,0xC0,0x66,0x0A,0xD9,
                                   0x0A,0x41,0x5D,0xAD,0x3A,0xBC,0x4F,0x38,0xE8,0xA1,0xA4,0x5E,0x24,0xDF,0x5A,0x7C,
                                   0x57,0x90,0x13,0xAF,0x66,0x8E,0x9D,0xCD,0x12,0xB6,0xA9,0xFF,0xF6,0xA2,0x3F,0x97,};
const unsigned char md5_dk_8[64]      = {0x22,0x60,0x34,0x73,0x1F,0xD1,0xAF,0x21,0x48,0x92,0xCA,0x46,0x2A,0x95,0xA1,0xFA,
                                   0xA2,0xF1,0x7A,0x3B,0xE0,0xC2,0x2C,0x04,0xDD,0xBC,0x6E,0x1C,0x13,0x59,0xEF,0x4D,
                                   0xF0,0x6B,0x9F,0x0D,0xA3,0x44,0x96,0xAE,0x8F,0x7D,0xD2,0xDE,0xFC,0xD1,0xDC,0xD3,
                                   0xAD,0xE6,0x6E,0x84,0xD6,0x1F,0x6B,0x30,0xCB,0x77,0x20,0xB3,0x75,0x4F,0xA7,0xD5,};
const unsigned char md5_dk_9[16]      = {0xEE,0xEB,0x6E,0xFD,0x18,0x80,0xAA,0xC9,0xCA,0x41,0x61,0xB7,0x16,0x24,0x84,0xD8,};
const unsigned char md5_dk_10[16]     = {0x3F,0x08,0x13,0x35,0xBB,0x7C,0x70,0x2A,0xB0,0x93,0x54,0x79,0x66,0x65,0xEB,0x06,};
const unsigned char md5_dk_11[16]     = {0x62,0x93,0xF8,0xE2,0xE1,0x7C,0x79,0x7A,0x28,0x5F,0x3E,0xCB,0x55,0xA8,0x3C,0x17,};

unsigned int pbkdf2_hmac_md5_test(void)
{
    /******************* inputs from RFC6070 ********************/
    if(pbkdf2_hmac_test_internal(HASH_MD5, password_1, 0, 8, salt_1, 4, 1, md5_dk_1, 16, "1"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_MD5, password_1, 0, 8, salt_1, 4, 2, md5_dk_2, 16, "2"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_MD5, password_1, 0, 8, salt_1, 4, 4096, md5_dk_3, 16, "3"))
        return 1;

#if 0
    if(pbkdf2_hmac_test_internal(HASH_MD5, password_1, 0, 8, salt_1, 4, 16777216, md5_dk_4, 16, "4"))
        return 1;
#endif

    if(pbkdf2_hmac_test_internal(HASH_MD5, password_2, 0, 24, salt_2, 36, 4096, md5_dk_5, 25, "5"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_MD5, password_3, 0, 9, salt_3, 5, 4096, md5_dk_6, 16, "6"))
        return 1;
    /************************************************************/

    /******************* inputs from RFC7914 ********************/
    if(pbkdf2_hmac_test_internal(HASH_MD5, password_4, 0, 6, salt_1, 4, 1, md5_dk_7, 64, "7"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_MD5, password_5, 0, 8, salt_5, 4, 80000, md5_dk_8, 64, "8"))
        return 1;
    /************************************************************/

    /*********************** corner cases ***********************/
    //password is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_MD5, password_1, 0, 0, salt_1, 4, 1, md5_dk_9, 16, "9"))
        return 1;

    //salt is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_MD5, password_1, 0, 8, salt_1, 0, 2, md5_dk_10, 16, "10"))
        return 1;

    //password and salt are both empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_MD5, password_1, 0, 0, salt_1, 0, 3, md5_dk_11, 16, "11"))
        return 1;
    /************************************************************/

    printf("\r\n");

    return 0;
}
#endif


#ifdef SUPPORT_HASH_SHA1
const unsigned char sha1_dk_1[20]     = {0x0C,0x60,0xC8,0x0F,0x96,0x1F,0x0E,0x71,0xF3,0xA9,0xB5,0x24,0xAF,0x60,0x12,0x06,0x2F,0xE0,0x37,0xA6};
const unsigned char sha1_dk_2[20]     = {0xEA,0x6C,0x01,0x4D,0xC7,0x2D,0x6F,0x8C,0xCD,0x1E,0xD9,0x2A,0xCE,0x1D,0x41,0xF0,0xD8,0xDE,0x89,0x57};
const unsigned char sha1_dk_3[20]     = {0x4B,0x00,0x79,0x01,0xB7,0x65,0x48,0x9A,0xBE,0xAD,0x49,0xD9,0x26,0xF7,0x21,0xD0,0x65,0xA4,0x29,0xC1};
const unsigned char sha1_dk_4[20]     = {0xEE,0xFE,0x3D,0x61,0xCD,0x4D,0xA4,0xE4,0xE9,0x94,0x5B,0x3D,0x6B,0xA2,0x15,0x8C,0x26,0x34,0xE9,0x84};
const unsigned char sha1_dk_5[25]     = {0x3D,0x2E,0xEC,0x4F,0xE4,0x1C,0x84,0x9B,0x80,0xC8,0xD8,0x36,0x62,0xC0,0xE4,0x4A,0x8B,0x29,0x1A,0x96,
                                   0x4C,0xF2,0xF0,0x70,0x38};
const unsigned char sha1_dk_6[16]     = {0x56,0xFA,0x6A,0xA7,0x55,0x48,0x09,0x9D,0xCC,0x37,0xD7,0xF0,0x34,0x25,0xE0,0xC3,};
const unsigned char sha1_dk_7[64]     = {0x69,0xF4,0x29,0x42,0x66,0x98,0xF0,0x69,0xA2,0xE1,0x7C,0xAA,0x33,0x43,0x64,0x5D,0xB3,0x05,0x76,0x5C,
                                   0x08,0xCE,0xE0,0xB8,0x68,0xED,0x42,0xA2,0x6E,0x6E,0x40,0x8A,0xDC,0x21,0x10,0xE1,0x4F,0x14,0xBB,0x36,
                                   0x22,0x79,0xC6,0x23,0xEF,0x8C,0xF0,0xCD,0x16,0x5A,0xBC,0x06,0x1A,0x87,0xD8,0x86,0x44,0x92,0xB2,0x02,
                                   0xCD,0x2D,0x85,0x9D,};
const unsigned char sha1_dk_8[64]     = {0x14,0xFA,0xF3,0xC9,0x43,0x86,0x0F,0x0F,0x54,0x3D,0x94,0xC4,0x49,0xAE,0x7F,0xE1,0x22,0x3A,0xFA,0x88,
                                   0x27,0x6C,0x7D,0x67,0x50,0x5D,0x71,0xF0,0x0B,0x1F,0x17,0xB8,0x40,0x7C,0x24,0x7D,0xA3,0x3C,0x64,0x0B,
                                   0xFA,0x9C,0x63,0x3E,0x38,0x40,0x5A,0x19,0xE9,0xAB,0xF2,0xDE,0xA7,0xBA,0x9F,0x00,0xE7,0xDC,0xB4,0x19,
                                   0x57,0x36,0x5D,0x8A,};
const unsigned char sha1_dk_9[20]     = {0xA3,0x3D,0xDD,0xC3,0x04,0x78,0x18,0x55,0x15,0x31,0x1F,0x87,0x52,0x89,0x5D,0x36,0xEA,0x43,0x63,0xA2,};
const unsigned char sha1_dk_10[20]    = {0x28,0x00,0x7D,0x55,0x46,0x1A,0xC8,0x0B,0xA1,0x3C,0xDA,0x42,0x2E,0x16,0x4B,0x8C,0x74,0x8C,0xE7,0x06,};
const unsigned char sha1_dk_11[20]    = {0x96,0xF4,0x42,0xC6,0x54,0x2B,0x98,0xB2,0x94,0x05,0xF5,0xB5,0xD2,0x30,0xE3,0x53,0x9E,0x1B,0x9D,0xFD,};
const unsigned char sha1_password[20] = {0x65,0x42,0x6B,0x58,0x51,0x54,0x66,0x75,0x42,0x71,0x70,0x27,0x63,0x54,0x63,0x61,0x72,0x26,0x67,0x2A};
const unsigned char sha1_salt[16]     = {0xA0,0x09,0xC1,0xA4,0x85,0x91,0x2C,0x6A,0xE6,0x30,0xD3,0xE7,0x44,0x24,0x0B,0x04};
const unsigned char sha1_dk_12[16]    = {0x17,0xEB,0x40,0x14,0xC8,0xC4,0x61,0xC3,0x00,0xE9,0xB6,0x15,0x18,0xB9,0xA1,0x8B,};

unsigned int pbkdf2_hmac_sha1_test(void)
{
    /**************** test vectors from RFC6070 *****************/
    if(pbkdf2_hmac_test_internal(HASH_SHA1, password_1, 0, 8, salt_1, 4, 1, sha1_dk_1, 20, "1"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA1, password_1, 0, 8, salt_1, 4, 2, sha1_dk_2, 20, "2"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA1, password_1, 0, 8, salt_1, 4, 4096, sha1_dk_3, 20, "3"))
        return 1;

#if 0
    if(pbkdf2_hmac_test_internal(HASH_SHA1, password_1, 0, 8, salt_1, 4, 16777216, sha1_dk_4, 20, "4"))
        return 1;
#endif

    if(pbkdf2_hmac_test_internal(HASH_SHA1, password_2, 0, 24, salt_2, 36, 4096, sha1_dk_5, 25, "5"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA1, password_3, 0, 9, salt_3, 5, 4096, sha1_dk_6, 16, "6"))
        return 1;
    /************************************************************/

    /******************* inputs from RFC7914 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SHA1, password_4, 0, 6, salt_1, 4, 1, sha1_dk_7, 64, "7"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA1, password_5, 0, 8, salt_5, 4, 80000, sha1_dk_8, 64, "8"))
        return 1;
    /************************************************************/

    /*********************** corner cases ***********************/
    //password is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA1, password_1, 0, 0, salt_1, 4, 1, sha1_dk_9, 20, "9"))
        return 1;

    //salt is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA1, password_1, 0, 8, salt_1, 0, 2, sha1_dk_10, 20, "10"))
        return 1;

    //password and salt are both empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA1, password_1, 0, 0, salt_1, 0, 3, sha1_dk_11, 20, "11"))
        return 1;
    /************************************************************/

    //this is from wikipedia
    if(pbkdf2_hmac_test_internal(HASH_SHA1, sha1_password, 0, 20, sha1_salt, 16, 1000, sha1_dk_12, 16, "12"))
        return 1;

    printf("\r\n");

    return 0;
}
#endif


#ifdef SUPPORT_HASH_SHA224
const unsigned char sha224_dk_1[28]   = {0x3C,0x19,0x8C,0xBD,0xB9,0x46,0x4B,0x78,0x57,0x96,0x6B,0xD0,0x5B,0x7B,0xC9,0x2B,
                                   0xC1,0xCC,0x4E,0x6E,0x63,0x15,0x5D,0x4E,0x49,0x05,0x57,0xFD,};
const unsigned char sha224_dk_2[28]   = {0x93,0x20,0x0F,0xFA,0x96,0xC5,0x77,0x6D,0x38,0xFA,0x10,0xAB,0xDF,0x8F,0x5B,0xFC,
                                   0x00,0x54,0xB9,0x71,0x85,0x13,0xDF,0x47,0x2D,0x23,0x31,0xD2,};
const unsigned char sha224_dk_3[28]   = {0x21,0x8C,0x45,0x3B,0xF9,0x06,0x35,0xBD,0x0A,0x21,0xA7,0x5D,0x17,0x27,0x03,0xFF,
                                   0x61,0x08,0xEF,0x60,0x3F,0x65,0xBB,0x82,0x1A,0xED,0xAD,0xE1,};
const unsigned char sha224_dk_4[28]   = {0xB4,0x99,0x25,0x18,0x4C,0xB4,0xB5,0x59,0xF3,0x65,0xE9,0x4F,0xCA,0xFC,0xD4,0xCD,
                                   0xB9,0xF7,0xAE,0xF4,0xA8,0xCA,0x8F,0xCB,0x4B,0xD1,0xEC,0x53,};
const unsigned char sha224_dk_5[40]   = {0x05,0x6C,0x4B,0xA4,0x38,0xDE,0xD9,0x1F,0xC1,0x4E,0x05,0x94,0xE6,0xF5,0x2B,0x87,
                                   0xE1,0xF3,0x69,0x0C,0x0D,0xC0,0xFB,0xC0,0x57,0x84,0xED,0x9A,0x75,0x4C,0xA7,0x80,
                                   0xE6,0xC0,0x17,0xE8,0x0C,0x8D,0xE2,0x78};
const unsigned char sha224_dk_6[16]   = {0x9B,0x40,0x11,0xB6,0x41,0xF4,0x0A,0x2A,0x50,0x0A,0x31,0xD4,0xA3,0x92,0xD1,0x5C,};
const unsigned char sha224_dk_7[64]   = {0xE5,0x5B,0xD7,0x7C,0xFC,0x18,0xB0,0x12,0xAC,0x63,0x62,0xE2,0x2D,0x7C,0xDF,0x77,
                                   0xC4,0xB0,0x38,0x79,0xA6,0xAF,0x51,0xFB,0xF0,0x04,0x5B,0xC3,0x2A,0x03,0xE7,0xF0,
                                   0xD8,0x29,0xD2,0x6B,0x76,0x5B,0xFF,0x0C,0xA5,0x87,0x3E,0x07,0xA8,0xE8,0x58,0x04,
                                   0xFF,0x4A,0x17,0x68,0x3E,0xD7,0x06,0x13,0x0D,0x51,0x65,0x74,0x56,0xBC,0x0E,0xBD,};
const unsigned char sha224_dk_8[64]   = {0xBE,0xBB,0xDF,0x80,0x9D,0x53,0xFC,0x84,0x53,0x1D,0x0A,0xBE,0x06,0x67,0x9A,0x8C,
                                   0x85,0x26,0xFD,0xE4,0x7B,0x47,0x24,0x56,0x34,0x18,0x69,0x08,0x33,0x58,0x57,0x33,
                                   0x4A,0x75,0x78,0x54,0x3F,0x92,0x41,0x72,0x6D,0x84,0x5E,0xE8,0xE5,0x75,0x10,0x5E,
                                   0x4A,0x73,0x3B,0x5D,0xCA,0xEF,0xA7,0x56,0x0A,0xF3,0xD0,0x28,0xEC,0xCF,0x95,0x93,};
const unsigned char sha224_dk_9[28]   = {0x35,0xCC,0x2B,0x82,0x1D,0xF1,0x41,0xE4,0x34,0x6A,0xE6,0x18,0x36,0xAC,0x2C,0x25,
                                   0xA6,0x53,0xC0,0x2A,0x79,0x9E,0x31,0x13,0xC4,0xF6,0x76,0x9D,};
const unsigned char sha224_dk_10[28]  = {0xCC,0x1A,0xA4,0x29,0xC8,0x6C,0x74,0x34,0x38,0xBB,0xD9,0x23,0xD9,0x17,0xAA,0x12,
                                   0x77,0xF3,0xB9,0x0D,0x86,0xD3,0x6E,0x56,0xFB,0xB7,0x1F,0x77,};
const unsigned char sha224_dk_11[28]  = {0x19,0x1C,0xF8,0x76,0x28,0x67,0xD5,0x04,0xC8,0x10,0x0F,0x46,0x50,0x73,0x95,0xEA,
                                   0x33,0x4F,0x09,0x54,0x6D,0x4B,0x03,0x34,0xAF,0x74,0x20,0x56,};

unsigned int pbkdf2_hmac_sha224_test(void)
{
    /******************* inputs from RFC6070 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SHA224, password_1, 0, 8, salt_1, 4, 1, sha224_dk_1, 28, "1"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA224, password_1, 0, 8, salt_1, 4, 2, sha224_dk_2, 28, "2"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA224, password_1, 0, 8, salt_1, 4, 4096, sha224_dk_3, 28, "3"))
        return 1;

#if 0
    if(pbkdf2_hmac_test_internal(HASH_SHA256, password_1, 0, 8, salt_1, 4, 16777216, sha224_dk_4, 28, "4"))
        return 1;
#endif

    if(pbkdf2_hmac_test_internal(HASH_SHA224, password_2, 0, 24, salt_2, 36, 4096, sha224_dk_5, 40, "5"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA224, password_3, 0, 9, salt_3, 5, 4096, sha224_dk_6, 16, "6"))
        return 1;
    /************************************************************/

    /******************* inputs from RFC7914 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SHA224, password_4, 0, 6, salt_1, 4, 1, sha224_dk_7, 64, "7"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA224, password_5, 0, 8, salt_5, 4, 80000, sha224_dk_8, 64, "8"))
        return 1;
    /************************************************************/

    /*********************** corner cases ***********************/
    //password is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA224, password_1, 0, 0, salt_1, 4, 1, sha224_dk_9, 28, "9"))
        return 1;

    //salt is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA224, password_1, 0, 8, salt_1, 0, 2, sha224_dk_10, 28, "10"))
        return 1;

    //password and salt are both empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA224, password_1, 0, 0, salt_1, 0, 3, sha224_dk_11, 28, "11"))
        return 1;
    /************************************************************/

    printf("\r\n");

    return 0;
}
#endif


#ifdef SUPPORT_HASH_SHA256
const unsigned char sha256_dk_1[32]   = {0x12,0x0F,0xB6,0xCF,0xFC,0xF8,0xB3,0x2C,0x43,0xE7,0x22,0x52,0x56,0xC4,0xF8,0x37,
                                   0xA8,0x65,0x48,0xC9,0x2C,0xCC,0x35,0x48,0x08,0x05,0x98,0x7C,0xB7,0x0B,0xE1,0x7B};
const unsigned char sha256_dk_2[32]   = {0xAE,0x4D,0x0C,0x95,0xAF,0x6B,0x46,0xD3,0x2D,0x0A,0xDF,0xF9,0x28,0xF0,0x6D,0xD0,
                                   0x2A,0x30,0x3F,0x8E,0xF3,0xC2,0x51,0xDF,0xD6,0xE2,0xD8,0x5A,0x95,0x47,0x4C,0x43};
const unsigned char sha256_dk_3[32]   = {0xC5,0xE4,0x78,0xD5,0x92,0x88,0xC8,0x41,0xAA,0x53,0x0D,0xB6,0x84,0x5C,0x4C,0x8D,
                                   0x96,0x28,0x93,0xA0,0x01,0xCE,0x4E,0x11,0xA4,0x96,0x38,0x73,0xAA,0x98,0x13,0x4A};
const unsigned char sha256_dk_4[32]   = {0xCF,0x81,0xC6,0x6F,0xE8,0xCF,0xC0,0x4D,0x1F,0x31,0xEC,0xB6,0x5D,0xAB,0x40,0x89,
                                   0xF7,0xF1,0x79,0xE8,0x9B,0x3B,0x0B,0xCB,0x17,0xAD,0x10,0xE3,0xAC,0x6E,0xBA,0x46};
const unsigned char sha256_dk_5[40]   = {0x34,0x8C,0x89,0xDB,0xCB,0xD3,0x2B,0x2F,0x32,0xD8,0x14,0xB8,0x11,0x6E,0x84,0xCF,
                                   0x2B,0x17,0x34,0x7E,0xBC,0x18,0x00,0x18,0x1C,0x4E,0x2A,0x1F,0xB8,0xDD,0x53,0xE1,
                                   0xC6,0x35,0x51,0x8C,0x7D,0xAC,0x47,0xE9};
const unsigned char sha256_dk_6[16]   = {0x89,0xB6,0x9D,0x05,0x16,0xF8,0x29,0x89,0x3C,0x69,0x62,0x26,0x65,0x0A,0x86,0x87,};
const unsigned char sha256_dk_7[64]   = {0x55,0xAC,0x04,0x6E,0x56,0xE3,0x08,0x9F,0xEC,0x16,0x91,0xC2,0x25,0x44,0xB6,0x05,
                                   0xF9,0x41,0x85,0x21,0x6D,0xDE,0x04,0x65,0xE6,0x8B,0x9D,0x57,0xC2,0x0D,0xAC,0xBC,
                                   0x49,0xCA,0x9C,0xCC,0xF1,0x79,0xB6,0x45,0x99,0x16,0x64,0xB3,0x9D,0x77,0xEF,0x31,
                                   0x7C,0x71,0xB8,0x45,0xB1,0xE3,0x0B,0xD5,0x09,0x11,0x20,0x41,0xD3,0xA1,0x97,0x83,};
const unsigned char sha256_dk_8[64]   = {0x4D,0xDC,0xD8,0xF6,0x0B,0x98,0xBE,0x21,0x83,0x0C,0xEE,0x5E,0xF2,0x27,0x01,0xF9,
                                   0x64,0x1A,0x44,0x18,0xD0,0x4C,0x04,0x14,0xAE,0xFF,0x08,0x87,0x6B,0x34,0xAB,0x56,
                                   0xA1,0xD4,0x25,0xA1,0x22,0x58,0x33,0x54,0x9A,0xDB,0x84,0x1B,0x51,0xC9,0xB3,0x17,
                                   0x6A,0x27,0x2B,0xDE,0xBB,0xA1,0xD0,0x78,0x47,0x8F,0x62,0xB3,0x97,0xF3,0x3C,0x8D,};
const unsigned char sha256_dk_9[32]   = {0xF1,0x35,0xC2,0x79,0x93,0xBA,0xF9,0x87,0x73,0xC5,0xCD,0xB4,0x0A,0x57,0x06,0xCE,
                                   0x6A,0x34,0x5C,0xDE,0x61,0xB0,0x00,0xA6,0x78,0x58,0x65,0x0C,0xD6,0xA3,0x24,0xD7,};
const unsigned char sha256_dk_10[32]  = {0x8B,0xC2,0xF9,0x16,0x7A,0x81,0xCD,0xCF,0xAD,0x12,0x35,0xCD,0x90,0x47,0xF1,0x13,
                                   0x62,0x71,0xC1,0xF9,0x78,0xFC,0xFC,0xB3,0x5E,0x22,0xDB,0xEA,0xFA,0x46,0x34,0xF6,};
const unsigned char sha256_dk_11[32]  = {0xB3,0x72,0x79,0x64,0x54,0xD3,0x7A,0xC0,0x42,0xA1,0x95,0xB6,0x2E,0xEB,0x7C,0xFE,
                                   0xD3,0x8D,0xDD,0x92,0xF7,0x57,0xE2,0xFE,0xE0,0x42,0x10,0xDF,0x03,0xAC,0x1D,0x2E,};

unsigned int pbkdf2_hmac_sha256_test(void)
{
    /******************* inputs from RFC6070 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SHA256, password_1, 0, 8, salt_1, 4, 1, sha256_dk_1, 32, "1"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA256, password_1, 0, 8, salt_1, 4, 2, sha256_dk_2, 32, "2"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA256, password_1, 0, 8, salt_1, 4, 4096, sha256_dk_3, 32, "3"))
        return 1;

#if 0
    if(pbkdf2_hmac_test_internal(HASH_SHA256, password_1, 0, 8, salt_1, 4, 16777216, sha256_dk_4, 32, "4"))
        return 1;
#endif

    if(pbkdf2_hmac_test_internal(HASH_SHA256, password_2, 0, 24, salt_2, 36, 4096, sha256_dk_5, 40, "5"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA256, password_3, 0, 9, salt_3, 5, 4096, sha256_dk_6, 16, "6"))
        return 1;
    /************************************************************/

    /**************** test vectors from RFC7914 *****************/
    if(pbkdf2_hmac_test_internal(HASH_SHA256, password_4, 0, 6, salt_1, 4, 1, sha256_dk_7, 64, "7"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA256, password_5, 0, 8, salt_5, 4, 80000, sha256_dk_8, 64, "8"))
        return 1;
    /************************************************************/

    /*********************** corner cases ***********************/
    //password is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA256, password_1, 0, 0, salt_1, 4, 1, sha256_dk_9, 32, "9"))
        return 1;

    //salt is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA256, password_1, 0, 8, salt_1, 0, 2, sha256_dk_10, 32, "10"))
        return 1;

    //password and salt are both empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA256, password_1, 0, 0, salt_1, 0, 3, sha256_dk_11, 32, "11"))
        return 1;
    /************************************************************/

    printf("\r\n");

    return 0;
}
#endif


#ifdef SUPPORT_HASH_SHA384
const unsigned char sha384_dk_1[48]   = {0xC0,0xE1,0x4F,0x06,0xE4,0x9E,0x32,0xD7,0x3F,0x9F,0x52,0xDD,0xF1,0xD0,0xC5,0xC7,
                                   0x19,0x16,0x09,0x23,0x36,0x31,0xDA,0xDD,0x76,0xA5,0x67,0xDB,0x42,0xB7,0x86,0x76,
                                   0xB3,0x8F,0xC8,0x00,0xCC,0x53,0xDD,0xB6,0x42,0xF5,0xC7,0x44,0x42,0xE6,0x2B,0xE4};
const unsigned char sha384_dk_2[48]   = {0x54,0xF7,0x75,0xC6,0xD7,0x90,0xF2,0x19,0x30,0x45,0x91,0x62,0xFC,0x53,0x5D,0xBF,
                                   0x04,0xA9,0x39,0x18,0x51,0x27,0x01,0x6A,0x04,0x17,0x6A,0x07,0x30,0xC6,0xF1,0xF4,
                                   0xFB,0x48,0x83,0x2A,0xD1,0x26,0x1B,0xAA,0xDD,0x2C,0xED,0xD5,0x08,0x14,0xB1,0xC8};
const unsigned char sha384_dk_3[48]   = {0x55,0x97,0x26,0xBE,0x38,0xDB,0x12,0x5B,0xC8,0x5E,0xD7,0x89,0x5F,0x6E,0x3C,0xF5,
                                   0x74,0xC7,0xA0,0x1C,0x08,0x0C,0x34,0x47,0xDB,0x1E,0x8A,0x76,0x76,0x4D,0xEB,0x3C,
                                   0x30,0x7B,0x94,0x85,0x3F,0xBE,0x42,0x4F,0x64,0x88,0xC5,0xF4,0xF1,0x28,0x96,0x26};
const unsigned char sha384_dk_4[48]   = {0xA7,0xFD,0xB3,0x49,0xBA,0x2B,0xFA,0x6B,0xF6,0x47,0xBB,0x01,0x61,0xBA,0xE1,0x32,
                                   0x0D,0xF2,0x7E,0x64,0x0A,0x04,0xE8,0xF1,0x11,0x48,0xC8,0x12,0x29,0xE2,0x13,0x1A,
                                   0xF1,0x79,0xC3,0xB3,0x42,0x3B,0x38,0xAB,0xF7,0x63,0xDF,0xE2,0x08,0xC1,0xFB,0x67};
const unsigned char sha384_dk_5[60]   = {0x81,0x91,0x43,0xAD,0x66,0xDF,0x9A,0x55,0x25,0x59,0xB9,0xE1,0x31,0xC5,0x2A,0xE6,
                                   0xC5,0xC1,0xB0,0xEE,0xD1,0x8F,0x4D,0x28,0x3B,0x8C,0x5C,0x9E,0xAE,0xB9,0x2B,0x39,
                                   0x2C,0x14,0x7C,0xC2,0xD2,0x86,0x9D,0x58,0xFF,0xE2,0xF7,0xDA,0x13,0xD1,0x5F,0x8D,
                                   0x92,0x57,0x21,0xF0,0xED,0x1A,0xFA,0xFA,0x24,0x48,0x0D,0x55};
const unsigned char sha384_dk_6[16]   = {0xA3,0xF0,0x0A,0xC8,0x65,0x7E,0x09,0x5F,0x8E,0x08,0x23,0xD2,0x32,0xFC,0x60,0xB3,};
const unsigned char sha384_dk_7[128]  = {0xCD,0x34,0x43,0x72,0x3A,0x41,0xCF,0x14,0x60,0xCC,0xA9,0xEF,0xEE,0xDE,0x42,0x8A,
                                   0x88,0x98,0xA8,0x2D,0x2A,0xD4,0xD1,0xFC,0x5C,0xCA,0x08,0xED,0x3F,0x4D,0x3C,0xB4,
                                   0x7A,0x62,0xA7,0x0B,0x3C,0xB9,0xCE,0x65,0xDC,0xBF,0xB9,0xFB,0x9D,0x42,0x50,0x27,
                                   0xA8,0xBE,0x69,0xB5,0x3E,0x2A,0x22,0x67,0x4B,0x09,0x39,0xE5,0xE0,0xA6,0x82,0xF7,
                                   0x6D,0x21,0xF4,0x49,0xAD,0x18,0x45,0x62,0xA3,0xBC,0x4C,0x51,0x9B,0x4D,0x04,0x8D,
                                   0xE6,0xD8,0xE0,0x99,0x9F,0xB8,0x87,0x70,0xF9,0x5E,0x40,0x18,0x5E,0x19,0xFC,0x8B,
                                   0x68,0x76,0x74,0x17,0xCC,0xC0,0x64,0xF4,0x7A,0x45,0x5D,0x04,0x5B,0x3B,0xAF,0xDA,
                                   0x7E,0x81,0xB9,0x7A,0xD0,0xE4,0xC5,0x58,0x1A,0xF1,0xAA,0x27,0x87,0x1C,0xD5,0xE4};
const unsigned char sha384_dk_8[128]  = {0x11,0xC1,0x98,0x98,0x77,0x30,0xFA,0x11,0x34,0x58,0x05,0x3C,0xD5,0xCC,0x9B,0x51,
                                   0xD7,0x02,0x4A,0x35,0xF9,0x13,0x4F,0x1E,0xE8,0x74,0x09,0x23,0xC9,0x01,0xAA,0xB2,
                                   0x3B,0xBA,0xEA,0x43,0x68,0x69,0x81,0xB6,0xE6,0xA9,0xF4,0x13,0x0A,0x14,0x01,0xDA,
                                   0xEE,0xEC,0x74,0x06,0x02,0x46,0xEB,0xAC,0x95,0x8F,0x3C,0xFC,0x3C,0x65,0x57,0x9B,
                                   0x6E,0x3D,0x08,0xB9,0x4A,0xDE,0x5F,0xC2,0x57,0xA6,0x90,0x2A,0x0A,0x16,0x64,0xB8,
                                   0xDB,0xD5,0xA8,0xAE,0x2A,0xF7,0x04,0x38,0x93,0x1D,0x3F,0x36,0x79,0xAB,0xFF,0xC7,
                                   0xA1,0x77,0x70,0x58,0x2F,0x1E,0xE4,0x13,0xCC,0x0D,0x99,0x14,0xCE,0x5F,0x81,0x43,
                                   0xC8,0xA7,0xDC,0x9C,0x43,0xFB,0xC3,0x1E,0x3D,0x41,0xB2,0x03,0x0F,0xB7,0x3C,0x02};
const unsigned char sha384_dk_9[48]   = {0x0F,0x2E,0x98,0xF6,0x83,0x44,0x69,0xC6,0xDD,0x38,0xA3,0x48,0x4E,0x8E,0xBA,0x4D,
                                   0xDF,0xA8,0x8B,0x64,0x40,0x35,0x80,0x24,0xE1,0xBF,0x86,0x5A,0xD1,0x55,0x40,0x1A,
                                   0x28,0xF7,0x3A,0x26,0xEA,0x1D,0x00,0xB8,0x52,0x79,0xCC,0xFD,0xAF,0x83,0x1C,0x07};
const unsigned char sha384_dk_10[48]  = {0x14,0x8D,0x78,0x4F,0xDD,0x1F,0x85,0x84,0x9C,0x48,0x96,0x84,0x00,0x31,0xFF,0x82,
                                   0x31,0x82,0x33,0x59,0xD0,0x6D,0x5D,0xB4,0x3F,0xB7,0x4A,0x8F,0x1D,0x5E,0x92,0x63,
                                   0x09,0x97,0xED,0x31,0x31,0x71,0xDC,0x45,0x73,0x5E,0xC6,0xD7,0x8D,0xD5,0xD2,0xDD,};
const unsigned char sha384_dk_11[48]  = {0xC3,0xA7,0x02,0xCB,0x4F,0xAA,0xAB,0xD1,0x11,0xE2,0xB6,0x4F,0xD7,0x85,0x1A,0xA2,
                                   0x78,0x34,0x87,0x5D,0x34,0x34,0x1D,0x0E,0x3B,0x36,0x25,0x17,0x55,0x71,0xF8,0x7C,
                                   0x35,0xD0,0x9B,0x9D,0x79,0x1F,0x77,0xB3,0xC5,0x2F,0xF2,0x65,0xEE,0xC2,0x08,0xCF,};

unsigned int pbkdf2_hmac_sha384_test(void)
{
    /******************* inputs from RFC6070 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SHA384, password_1, 0, 8, salt_1, 4, 1, sha384_dk_1, 48, "1"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA384, password_1, 0, 8, salt_1, 4, 2, sha384_dk_2, 48, "2"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA384, password_1, 0, 8, salt_1, 4, 4096, sha384_dk_3, 48, "3"))
        return 1;

#if 0
    if(pbkdf2_hmac_test_internal(HASH_SHA384, password_1, 0, 8, salt_1, 4, 16777216, sha384_dk_4, 48, "4"))
        return 1;
#endif

    if(pbkdf2_hmac_test_internal(HASH_SHA384, password_2, 0, 24, salt_2, 36, 4096, sha384_dk_5, 60, "5"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA384, password_3, 0, 9, salt_3, 5, 4096, sha384_dk_6, 16, "6"))
        return 1;
    /************************************************************/

    /******************* inputs from RFC7914 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SHA384, password_4, 0, 6, salt_1, 4, 1, sha384_dk_7, 128, "7"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA384, password_5, 0, 8, salt_5, 4, 80000, sha384_dk_8, 128, "8"))
        return 1;
    /************************************************************/

    /*********************** corner cases ***********************/
    //password is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA384, password_1, 0, 0, salt_1, 4, 1, sha384_dk_9, 48, "9"))
        return 1;

    //salt is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA384, password_1, 0, 8, salt_1, 0, 2, sha384_dk_10, 48, "10"))
        return 1;

    //password and salt are both empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA384, password_1, 0, 0, salt_1, 0, 3, sha384_dk_11, 48, "11"))
        return 1;
    /************************************************************/

    printf("\r\n");

    return 0;
}
#endif


#ifdef SUPPORT_HASH_SHA512_224
const unsigned char sha512_224_dk_1[28] = {0xB3,0x4A,0xB6,0x26,0x27,0x6A,0x61,0xCE,0x19,0xD2,0xEC,0xB4,0xC7,0xE1,0x5F,0x81,
                                     0x98,0xA2,0x98,0x9A,0xBD,0x74,0xAD,0xE6,0x1C,0xD6,0xB1,0x17,};
const unsigned char sha512_224_dk_2[28] = {0xB8,0x87,0x8A,0xC5,0xE4,0x50,0x9C,0x16,0x5C,0x1B,0x50,0x89,0x61,0xFA,0x3C,0x3A,
                                     0xFC,0xEF,0x3F,0x37,0xB7,0xB0,0x81,0x87,0x4E,0x71,0x8D,0x8D,};
const unsigned char sha512_224_dk_3[28] = {0xED,0x54,0xAF,0x69,0x9C,0xC3,0x07,0xE0,0x89,0x65,0x09,0x8B,0xDA,0x5F,0xF4,0xE4,
                                     0x1E,0xA1,0x93,0x1F,0x46,0xDA,0x77,0x1C,0x1E,0xA9,0x12,0x8E,};
const unsigned char sha512_224_dk_4[28] = {0xCA,0x62,0x62,0xFA,0x52,0xEA,0xD9,0x88,0x78,0x86,0x7E,0x13,0x75,0xD0,0xA5,0x8A,
                                     0x8A,0xE0,0x4D,0x00,0xDA,0x69,0xD0,0xAE,0xE5,0x5A,0x88,0xED,};
const unsigned char sha512_224_dk_5[40] = {0x57,0x3D,0xF9,0x67,0x62,0xEA,0x7D,0xA4,0xF7,0x12,0x31,0x85,0x9C,0xA2,0x82,0xEF,
                                     0x48,0x27,0x64,0xAD,0x96,0x71,0xC5,0x27,0x5C,0x32,0x72,0xFE,0x6A,0xE9,0x4D,0x28,
                                     0x5A,0x57,0x09,0xD1,0x08,0x0F,0xD6,0xD8};
const unsigned char sha512_224_dk_6[16] = {0xC8,0x66,0x4F,0x7A,0xDD,0xE2,0xC3,0x07,0x3C,0x96,0xD4,0x8A,0x16,0xEC,0x50,0x7B,};
const unsigned char sha512_224_dk_7[64] = {0x2C,0xE4,0x99,0xB4,0x59,0xAD,0xB2,0xFE,0xC9,0x79,0x64,0xFB,0x6C,0x1C,0x2A,0xB7,
                                     0x00,0x40,0x13,0x95,0x8C,0xF7,0x86,0x49,0x95,0x1D,0xFB,0x69,0x45,0x63,0x38,0x5D,
                                     0x02,0xBA,0xA4,0x26,0x34,0xC7,0x45,0xAC,0xFC,0xD9,0x75,0x02,0xA0,0x3E,0xD5,0x38,
                                     0x8A,0xC5,0xF1,0x61,0x3E,0xC2,0x83,0xB4,0xF2,0x1C,0xBA,0x6E,0x58,0x32,0x55,0x85,};
const unsigned char sha512_224_dk_8[64] = {0x8E,0x5B,0x3E,0xED,0x84,0x27,0xBD,0x6A,0x87,0x1E,0x37,0xAA,0xD9,0xC6,0xCA,0xE8,
                                     0x3A,0x4E,0x02,0x96,0xD1,0x3C,0x72,0xBC,0xF8,0x38,0xCC,0xAF,0xCE,0xA6,0x0F,0x8A,
                                     0x48,0xBB,0xB1,0xF8,0xA8,0x81,0x22,0x60,0xE0,0x3A,0x17,0xD7,0x8A,0xDE,0xCE,0x79,
                                     0x51,0xEB,0x6C,0x81,0x4F,0x85,0x2E,0x3D,0x0B,0x9C,0xD5,0xDC,0x66,0xF4,0xD7,0xEA,};
const unsigned char sha512_224_dk_9[28] = {0x3F,0xB5,0x1E,0x5F,0xB6,0x11,0x2E,0x7F,0xA9,0x46,0x8F,0x93,0xB7,0x10,0xA0,0x76,
                                     0x4B,0x6D,0x31,0x32,0xF5,0xD4,0xD7,0xCE,0xF0,0x63,0x3D,0x19,};
const unsigned char sha512_224_dk_10[28]= {0x9C,0x15,0x10,0x4D,0x11,0xA8,0xA1,0x2D,0x1D,0xE6,0x5C,0x27,0xD5,0x96,0xB3,0x48,
                                     0x70,0x08,0x79,0x29,0xDC,0x43,0x12,0xF6,0xDC,0x1B,0xCE,0xF2,};
const unsigned char sha512_224_dk_11[28]= {0xAC,0x5D,0x80,0x8F,0x95,0x32,0x0F,0x05,0x5D,0x2F,0x8F,0x12,0xE3,0x04,0x60,0x00,
                                     0xEC,0x9F,0xF9,0xAD,0xDA,0x63,0x6D,0x36,0x5A,0xBE,0x03,0xFB,};

unsigned int pbkdf2_hmac_sha512_224_test(void)
{
    /******************* inputs from RFC6070 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SHA512_224, password_1, 0, 8, salt_1, 4, 1, sha512_224_dk_1, 28, "1"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA512_224, password_1, 0, 8, salt_1, 4, 2, sha512_224_dk_2, 28, "2"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA512_224, password_1, 0, 8, salt_1, 4, 4096, sha512_224_dk_3, 28, "3"))
        return 1;

#if 0
    if(pbkdf2_hmac_test_internal(HASH_SHA512_224, password_1, 0, 8, salt_1, 4, 16777216, sha512_224_dk_4, 28, "4"))
        return 1;
#endif

    if(pbkdf2_hmac_test_internal(HASH_SHA512_224, password_2, 0, 24, salt_2, 36, 4096, sha512_224_dk_5, 40, "5"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA512_224, password_3, 0, 9, salt_3, 5, 4096, sha512_224_dk_6, 16, "6"))
        return 1;
    /************************************************************/

    /******************* inputs from RFC7914 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SHA512_224, password_4, 0, 6, salt_1, 4, 1, sha512_224_dk_7, 64, "7"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA512_224, password_5, 0, 8, salt_5, 4, 80000, sha512_224_dk_8, 64, "8"))
        return 1;
    /************************************************************/

    /*********************** corner cases ***********************/
    //password is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA512_224, password_1, 0, 0, salt_1, 4, 1, sha512_224_dk_9, 28, "9"))
        return 1;

    //salt is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA512_224, password_1, 0, 8, salt_1, 0, 2, sha512_224_dk_10, 28, "10"))
        return 1;

    //password and salt are both empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA512_224, password_1, 0, 0, salt_1, 0, 3, sha512_224_dk_11, 28, "11"))
        return 1;
    /************************************************************/

    printf("\r\n");

    return 0;
}
#endif


#ifdef SUPPORT_HASH_SHA512_256
const unsigned char sha512_256_dk_1[32] = {0x4B,0x6A,0x63,0x11,0x7D,0x3E,0xC0,0x03,0x26,0x24,0x61,0x60,0x82,0xC1,0xC1,0x91,
                                     0x2F,0x56,0xFA,0x5F,0x0C,0x1F,0x94,0x57,0x4D,0x51,0x5E,0x20,0xE5,0xDD,0xD7,0x4A};
const unsigned char sha512_256_dk_2[32] = {0xFC,0xFD,0x10,0x8C,0x99,0xCC,0x88,0x8E,0xC0,0xAF,0x9F,0x18,0x48,0x85,0xAF,0xF5,
                                     0xF0,0x2D,0x19,0xA9,0x56,0xAF,0xAD,0x9C,0xCE,0xA4,0xD5,0x6A,0x48,0x2B,0x85,0x1B};
const unsigned char sha512_256_dk_3[32] = {0xF2,0xFB,0xE5,0xF8,0xEC,0x36,0x18,0xBB,0x14,0x52,0x79,0xA8,0xC6,0xA8,0xDF,0xA4,
                                     0x76,0xC2,0x82,0xA3,0xED,0x53,0xD8,0xC2,0x57,0xD5,0x1C,0xE0,0x21,0xD3,0x87,0x7D};
const unsigned char sha512_256_dk_4[32] = {0x25,0xF5,0xD3,0x8F,0x4B,0x39,0xD9,0x2B,0x43,0x6E,0x52,0xF8,0xA9,0x2F,0xB6,0x9D,
                                     0xF7,0x6F,0xB0,0x14,0x89,0x85,0xC2,0xF9,0x35,0xD6,0x26,0xD4,0xF0,0xC6,0x90,0x37};
const unsigned char sha512_256_dk_5[40] = {0x31,0xCF,0x94,0xE3,0xD8,0xE3,0x6A,0xA1,0x8D,0x40,0xAD,0x92,0x65,0x4A,0xB8,0x0F,
                                     0x50,0x0E,0xD7,0xFB,0x57,0x5A,0x22,0x15,0x54,0x7D,0xB6,0xF8,0x2D,0xD2,0x27,0xED,
                                     0x0F,0x41,0x21,0x5E,0x8F,0x9B,0xB9,0x76};
const unsigned char sha512_256_dk_6[16] = {0x7C,0x7B,0xD6,0x94,0x76,0x8C,0x9D,0xEA,0x9A,0xDB,0x86,0x23,0x30,0x8D,0xC0,0xA4,};
const unsigned char sha512_256_dk_7[64] = {0x3A,0x68,0xC0,0x01,0xBD,0xF2,0x5C,0xB6,0xA8,0xFB,0x18,0x7B,0xC2,0x54,0xDA,0xBC,
                                     0xC6,0x1D,0xAF,0x61,0xC7,0xCE,0x86,0x51,0xE9,0x74,0xE0,0x4C,0xF3,0x04,0xC8,0xF6,
                                     0x23,0xF3,0xBB,0x83,0x72,0xBD,0xF3,0x07,0xE5,0x69,0x5A,0xED,0x5A,0xA0,0xEE,0xD2,
                                     0x78,0x17,0xB3,0xE1,0x11,0xF2,0x5C,0x6C,0xD2,0xDE,0xCD,0x2C,0x54,0xDD,0x4A,0x3F,};
const unsigned char sha512_256_dk_8[64] = {0x8A,0x06,0x2F,0x25,0x7C,0xE4,0x1E,0xEC,0x8E,0xA0,0x27,0x3A,0x06,0x81,0xAB,0xD1,
                                     0x52,0x2F,0xEA,0x35,0x40,0x7F,0xE2,0x93,0x93,0xE6,0x18,0x7F,0xED,0x87,0x50,0x64,
                                     0x78,0x10,0x90,0x28,0x03,0x83,0xFC,0x9E,0x4D,0x17,0x51,0x49,0xCD,0xE2,0x9A,0x15,
                                     0x53,0x15,0x5A,0xBE,0xD7,0xC3,0xF2,0xC5,0xFB,0x51,0xE1,0xFE,0x20,0x3A,0xB7,0x65,};
const unsigned char sha512_256_dk_9[32] = {0xC0,0x7E,0xB1,0xCC,0x74,0x8E,0x7C,0x43,0xA7,0xCA,0x1F,0x7A,0xA7,0x3B,0x32,0x5A,
                                     0x56,0x81,0x05,0x09,0x8D,0x7A,0x35,0x8C,0x68,0x2A,0x90,0xCD,0xA4,0x85,0x78,0xEB,};
const unsigned char sha512_256_dk_10[32]= {0xF5,0x9A,0xEB,0xD4,0xA4,0x99,0xC5,0x85,0x8A,0xC4,0x03,0x68,0x0A,0xDD,0xDB,0x62,
                                     0xE1,0x26,0xDB,0xFA,0x80,0x97,0x65,0x41,0xE8,0x03,0x9F,0x5D,0xA3,0x44,0x0B,0x68,};
const unsigned char sha512_256_dk_11[32]= {0x5F,0x96,0x94,0x29,0xB5,0x8C,0x76,0xE8,0x2C,0x02,0xAC,0x09,0xBB,0x4D,0xB9,0xAC,
                                     0x71,0x51,0x4E,0x3F,0xF9,0xF4,0xCF,0xF8,0xB4,0x14,0xF2,0x5F,0x0F,0xC2,0x69,0xFB,};

unsigned int pbkdf2_hmac_sha512_256_test(void)
{
    /******************* inputs from RFC6070 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SHA512_256, password_1, 0, 8, salt_1, 4, 1, sha512_256_dk_1, 32, "1"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA512_256, password_1, 0, 8, salt_1, 4, 2, sha512_256_dk_2, 32, "2"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA512_256, password_1, 0, 8, salt_1, 4, 4096, sha512_256_dk_3, 32, "3"))
        return 1;

#if 0
    if(pbkdf2_hmac_test_internal(HASH_SHA512_256, password_1, 0, 8, salt_1, 4, 16777216, sha512_256_dk_4, 32, "4"))
        return 1;
#endif

    if(pbkdf2_hmac_test_internal(HASH_SHA512_256, password_2, 0, 24, salt_2, 36, 4096, sha512_256_dk_5, 40, "5"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA512_256, password_3, 0, 9, salt_3, 5, 4096, sha512_256_dk_6, 16, "6"))
        return 1;
    /************************************************************/

    /******************* inputs from RFC7914 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SHA512_256, password_4, 0, 6, salt_1, 4, 1, sha512_256_dk_7, 64, "7"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA512_256, password_5, 0, 8, salt_5, 4, 80000, sha512_256_dk_8, 64, "8"))
        return 1;
    /************************************************************/

    /*********************** corner cases ***********************/
    //password is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA512_256, password_1, 0, 0, salt_1, 4, 1, sha512_256_dk_9, 32, "9"))
        return 1;

    //salt is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA512_256, password_1, 0, 8, salt_1, 0, 2, sha512_256_dk_10, 32, "10"))
        return 1;

    //password and salt are both empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA512_256, password_1, 0, 0, salt_1, 0, 3, sha512_256_dk_11, 32, "11"))
        return 1;
    /************************************************************/

    printf("\r\n");

    return 0;
}
#endif


#ifdef SUPPORT_HASH_SHA512
const unsigned char sha512_dk_1[64]   = {0x86,0x7F,0x70,0xCF,0x1A,0xDE,0x02,0xCF,0xF3,0x75,0x25,0x99,0xA3,0xA5,0x3D,0xC4,
                                   0xAF,0x34,0xC7,0xA6,0x69,0x81,0x5A,0xE5,0xD5,0x13,0x55,0x4E,0x1C,0x8C,0xF2,0x52,
                                   0xC0,0x2D,0x47,0x0A,0x28,0x5A,0x05,0x01,0xBA,0xD9,0x99,0xBF,0xE9,0x43,0xC0,0x8F,
                                   0x05,0x02,0x35,0xD7,0xD6,0x8B,0x1D,0xA5,0x5E,0x63,0xF7,0x3B,0x60,0xA5,0x7F,0xCE};
const unsigned char sha512_dk_2[64]   = {0xE1,0xD9,0xC1,0x6A,0xA6,0x81,0x70,0x8A,0x45,0xF5,0xC7,0xC4,0xE2,0x15,0xCE,0xB6,
                                   0x6E,0x01,0x1A,0x2E,0x9F,0x00,0x40,0x71,0x3F,0x18,0xAE,0xFD,0xB8,0x66,0xD5,0x3C,
                                   0xF7,0x6C,0xAB,0x28,0x68,0xA3,0x9B,0x9F,0x78,0x40,0xED,0xCE,0x4F,0xEF,0x5A,0x82,
                                   0xBE,0x67,0x33,0x5C,0x77,0xA6,0x06,0x8E,0x04,0x11,0x27,0x54,0xF2,0x7C,0xCF,0x4E};
const unsigned char sha512_dk_3[64]   = {0xD1,0x97,0xB1,0xB3,0x3D,0xB0,0x14,0x3E,0x01,0x8B,0x12,0xF3,0xD1,0xD1,0x47,0x9E,
                                   0x6C,0xDE,0xBD,0xCC,0x97,0xC5,0xC0,0xF8,0x7F,0x69,0x02,0xE0,0x72,0xF4,0x57,0xB5,
                                   0x14,0x3F,0x30,0x60,0x26,0x41,0xB3,0xD5,0x5C,0xD3,0x35,0x98,0x8C,0xB3,0x6B,0x84,
                                   0x37,0x60,0x60,0xEC,0xD5,0x32,0xE0,0x39,0xB7,0x42,0xA2,0x39,0x43,0x4A,0xF2,0xD5};
const unsigned char sha512_dk_4[64]   = {0x61,0x80,0xA3,0xCE,0xAB,0xAB,0x45,0xCC,0x39,0x64,0x11,0x2C,0x81,0x1E,0x01,0x31,
                                   0xBC,0xA9,0x3A,0x35,0xD1,0x7E,0x83,0x3E,0xBC,0x22,0x1A,0x40,0xBD,0x75,0x8A,0xE8,
                                   0x32,0x88,0x02,0x89,0x6E,0x40,0xA5,0x4D,0x2B,0xE2,0x1C,0x7D,0xD8,0xE6,0x65,0xF6,
                                   0x49,0x0A,0xBB,0xBD,0xFF,0x6C,0x55,0x90,0xBC,0x65,0x6C,0x9D,0x0B,0x3D,0xAD,0x2A};
const unsigned char sha512_dk_5[70]   = {0x8C,0x05,0x11,0xF4,0xC6,0xE5,0x97,0xC6,0xAC,0x63,0x15,0xD8,0xF0,0x36,0x2E,0x22,
                                   0x5F,0x3C,0x50,0x14,0x95,0xBA,0x23,0xB8,0x68,0xC0,0x05,0x17,0x4D,0xC4,0xEE,0x71,
                                   0x11,0x5B,0x59,0xF9,0xE6,0x0C,0xD9,0x53,0x2F,0xA3,0x3E,0x0F,0x75,0xAE,0xFE,0x30,
                                   0x22,0x5C,0x58,0x3A,0x18,0x6C,0xD8,0x2B,0xD4,0xDA,0xEA,0x97,0x24,0xA3,0xD3,0xB8,
                                   0x04,0xF7,0x5B,0xDD,0x41,0x49};
const unsigned char sha512_dk_6[16]   = {0x9D,0x9E,0x9C,0x4C,0xD2,0x1F,0xE4,0xBE,0x24,0xD5,0xB8,0x24,0x4C,0x75,0x96,0x65};
const unsigned char sha512_dk_7[128]  = {0xC7,0x43,0x19,0xD9,0x94,0x99,0xFC,0x3E,0x90,0x13,0xAC,0xFF,0x59,0x7C,0x23,0xC5,
                                   0xBA,0xF0,0xA0,0xBE,0xC5,0x63,0x4C,0x46,0xB8,0x35,0x2B,0x79,0x3E,0x32,0x47,0x23,
                                   0xD5,0x5C,0xAA,0x76,0xB2,0xB2,0x5C,0x43,0x40,0x2D,0xCF,0xDC,0x06,0xCD,0xCF,0x66,
                                   0xF9,0x5B,0x7D,0x04,0x29,0x42,0x0B,0x39,0x52,0x00,0x06,0x74,0x9C,0x51,0xA0,0x4E,
                                   0xF3,0xEB,0x99,0xE5,0x76,0x61,0x73,0x95,0xA1,0x78,0xBA,0x33,0x21,0x47,0x93,0xE4,
                                   0x80,0x45,0x13,0x29,0x28,0xA9,0xE9,0xBF,0x26,0x61,0x76,0x9F,0xDC,0x66,0x8F,0x31,
                                   0x79,0x85,0x97,0xAA,0xF6,0xDA,0x70,0xDD,0x99,0x6A,0x81,0x01,0x97,0x26,0x08,0x4D,
                                   0x70,0xF1,0x52,0xBA,0xED,0x8A,0xAF,0xE2,0x22,0x7C,0x07,0x63,0x6C,0x6D,0xDE,0xCE};
const unsigned char sha512_dk_8[128]  = {0xE6,0x33,0x7D,0x6F,0xBE,0xB6,0x45,0xC7,0x94,0xD4,0xA9,0xB5,0xB7,0x5B,0x7B,0x30,
                                   0xDA,0xC9,0xAC,0x50,0x37,0x6A,0x91,0xDF,0x1F,0x44,0x60,0xF6,0x06,0x0D,0x5A,0xDD,
                                   0xB2,0xC1,0xFD,0x1F,0x84,0x40,0x9A,0xBA,0xCC,0x67,0xDE,0x7E,0xB4,0x05,0x6E,0x6B,
                                   0xB0,0x6C,0x2D,0x82,0xC3,0xEF,0x4C,0xCD,0x1B,0xDE,0xD0,0xF6,0x75,0xED,0x97,0xC6,
                                   0x5C,0x33,0xD3,0x9F,0x81,0x24,0x84,0x54,0x32,0x7A,0xA6,0xD0,0x3F,0xD0,0x49,0xFC,
                                   0x5C,0xBB,0x2B,0x5E,0x6D,0xAC,0x08,0xE8,0xAC,0xE9,0x96,0xCD,0xC9,0x60,0xB1,0xBD,
                                   0x45,0x30,0xB7,0xE7,0x54,0x77,0x3D,0x75,0xF6,0x7A,0x73,0x3F,0xDB,0x99,0xBA,0xF6,
                                   0x47,0x0E,0x42,0xFF,0xCB,0x75,0x3C,0x15,0xC3,0x52,0xD4,0x80,0x0F,0xB6,0xF9,0xD6};
const unsigned char sha512_dk_9[64]   = {0x00,0xEF,0x42,0xCD,0xBF,0xC9,0x8D,0x29,0xDB,0x20,0x97,0x66,0x08,0xE4,0x55,0x56,
                                   0x7F,0xDD,0xDF,0x14,0x1F,0x6E,0xB0,0x3B,0x5A,0x85,0xAD,0xDD,0x25,0x97,0x4F,0x5D,
                                   0x23,0x75,0xBD,0x50,0x82,0xB8,0x03,0xE8,0xF4,0xCF,0xA8,0x8A,0xE1,0xBD,0x25,0x25,
                                   0x6F,0xCB,0xDD,0xD2,0x31,0x86,0x76,0x56,0x6F,0xF2,0x79,0x77,0x92,0x30,0x2A,0xEE};
const unsigned char sha512_dk_10[64]  = {0x52,0xB0,0x5E,0x3B,0x51,0x89,0x3D,0x18,0x48,0x88,0x08,0xEC,0xE2,0xA6,0xB8,0xBD,
                                   0x4E,0xFC,0xD0,0x0E,0x48,0xDB,0x98,0x2B,0x03,0xE5,0x22,0x4D,0x87,0x2A,0xA5,0x23,
                                   0x1F,0x3E,0x77,0x84,0xE0,0x86,0x01,0xC8,0x0F,0x90,0xCE,0x44,0x09,0x9F,0x5B,0xE4,
                                   0x7C,0xFA,0xC9,0xA1,0x40,0xB7,0x78,0xB9,0x89,0xFF,0x57,0x46,0xB0,0x9D,0x97,0xAB};
const unsigned char sha512_dk_11[64]  = {0xBA,0x78,0xA2,0xC1,0x8F,0xE1,0xF3,0xCF,0xFF,0xAB,0xA0,0xF9,0x3C,0xCF,0x85,0xFC,
                                   0x34,0x2E,0xDF,0x2C,0x34,0xFF,0x43,0x9F,0x92,0x8C,0x46,0x4F,0xE0,0x4E,0xE9,0xC4,
                                   0x3D,0x65,0x16,0x59,0x42,0x82,0xBF,0x31,0xD9,0xB8,0x0B,0xA3,0x1D,0x67,0xE6,0xD0,
                                   0x76,0xCC,0x2E,0x45,0xBA,0x2D,0x5C,0x9D,0x6B,0xCA,0x44,0xF2,0x88,0x94,0x0C,0x5D};

unsigned int pbkdf2_hmac_sha512_test(void)
{
    /******************* inputs from RFC6070 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SHA512, password_1, 0, 8, salt_1, 4, 1, sha512_dk_1, 64, "1"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA512, password_1, 0, 8, salt_1, 4, 2, sha512_dk_2, 64, "2"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA512, password_1, 0, 8, salt_1, 4, 4096, sha512_dk_3, 64, "3"))
        return 1;

#if 0
    if(pbkdf2_hmac_test_internal(HASH_SHA512, password_1, 0, 8, salt_1, 4, 16777216, sha512_dk_4, 64, "4"))
        return 1;
#endif

    if(pbkdf2_hmac_test_internal(HASH_SHA512, password_2, 0, 24, salt_2, 36, 4096, sha512_dk_5, 70, "5"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA512, password_3, 0, 9, salt_3, 5, 4096, sha512_dk_6, 16, "6"))
        return 1;
    /************************************************************/

    /******************* inputs from RFC7914 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SHA512, password_4, 0, 6, salt_1, 4, 1, sha512_dk_7, 128, "7"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SHA512, password_5, 0, 8, salt_5, 4, 80000, sha512_dk_8, 128, "8"))
        return 1;
    /************************************************************/

    /*********************** corner cases ***********************/
    //password is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA512, password_1, 0, 0, salt_1, 4, 1, sha512_dk_9, 64, "9"))
        return 1;

    //salt is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA512, password_1, 0, 8, salt_1, 0, 2, sha512_dk_10, 64, "10"))
        return 1;

    //password and salt are both empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SHA512, password_1, 0, 0, salt_1, 0, 3, sha512_dk_11, 64, "11"))
        return 1;
    /************************************************************/

    printf("\r\n");

    return 0;
}
#endif


#ifdef SUPPORT_HASH_SM3
const unsigned char sm3_dk_1[32]      = {0x46,0x12,0xF9,0x22,0xA1,0xFD,0xCE,0xFA,0xF4,0x31,0x2F,0xC6,0xF8,0xF3,0x32,0x2B,
                                   0x48,0x9C,0xBF,0x24,0xF2,0xEA,0x36,0x1B,0x44,0xC2,0xBD,0x8F,0xA2,0xC6,0xDC,0xB0};
const unsigned char sm3_dk_2[32]      = {0xFE,0xE7,0x23,0xA2,0xBC,0x96,0x6E,0x11,0xDF,0xFB,0x66,0x13,0x3F,0x4E,0x8D,0xF5,
                                   0x77,0x38,0x3C,0x78,0xAD,0xE3,0x0E,0x32,0x98,0xED,0xBD,0x3E,0x54,0xED,0x85,0xB7};
const unsigned char sm3_dk_3[32]      = {0xB6,0xE8,0xF2,0x07,0x4C,0x87,0x43,0x2B,0x78,0xF6,0x2E,0x5C,0xED,0x98,0x0F,0xDF,
                                   0xF8,0x9E,0x86,0xAF,0x2F,0x69,0x3D,0xAB,0x16,0x38,0xE2,0xB3,0x68,0x30,0x45,0xDD};
const unsigned char sm3_dk_4[32]      = {0xA8,0xBD,0x50,0xAA,0xF3,0x69,0x81,0xBD,0x26,0x9A,0xC7,0xE8,0x85,0x68,0x38,0x08,
                                   0x8A,0x3D,0x0E,0xC4,0xA8,0x61,0xD1,0x0C,0x71,0x67,0x8D,0x04,0x66,0x60,0x15,0x6C};
const unsigned char sm3_dk_5[40]      = {0x3B,0x62,0x82,0xAC,0x85,0x19,0xF0,0x59,0xE4,0x65,0xAB,0xFF,0x0E,0xA3,0x7B,0x0D,
                                   0xBF,0xE6,0xC6,0x72,0xA7,0x6E,0x6B,0x80,0x53,0x12,0xD5,0x39,0x00,0xDB,0x63,0x07,
                                   0x32,0xCC,0xC1,0xA8,0x8F,0xA5,0x51,0x2A};
const unsigned char sm3_dk_6[16]      = {0x5F,0x93,0x6B,0x2E,0x35,0x6F,0x06,0xE2,0xBB,0x39,0x32,0x16,0x58,0x21,0x26,0x1C,};
const unsigned char sm3_dk_7[64]      = {0x30,0x53,0x0B,0x86,0x53,0x7C,0x85,0xEA,0x33,0xFB,0xE2,0x6E,0xE0,0xAD,0xEA,0x8D,
                                   0x8B,0xD9,0x1D,0x28,0x3B,0x87,0x93,0x6F,0x40,0x0E,0x83,0xAD,0x23,0x64,0xEF,0x26,
                                   0x32,0xCD,0x86,0x62,0x31,0x36,0x57,0xC0,0xE6,0x13,0x8F,0xFE,0xDA,0x2C,0x4E,0x68,
                                   0x05,0x86,0x9C,0xBD,0x80,0x30,0x18,0x88,0x58,0xD8,0xA6,0x75,0xAC,0x76,0x25,0x23,};
const unsigned char sm3_dk_8[64]      = {0x60,0x64,0xDD,0x50,0x56,0x8A,0x7C,0xD0,0xFA,0xB5,0x2A,0xEA,0x44,0x3F,0xD7,0x1C,
                                   0x6F,0x45,0x04,0x4F,0xD0,0xAE,0xDE,0x59,0x28,0x56,0x8B,0xE7,0x94,0x8A,0x34,0x69,
                                   0x9D,0x2B,0xFC,0xE8,0xA4,0x35,0x11,0x0F,0x55,0x1C,0xC0,0xE1,0x81,0x46,0x02,0xC0,
                                   0x3C,0x11,0xEC,0xB7,0xF9,0x51,0xFE,0xC5,0x06,0x02,0x85,0x0A,0x51,0x02,0xCB,0x88,};
const unsigned char sm3_dk_9[32]      = {0x83,0x88,0xDD,0x39,0x99,0xFB,0x0C,0xDE,0x40,0x64,0xA6,0xA3,0xF5,0xC1,0xA5,0x96,
                                   0x3B,0xF4,0xE2,0xCD,0x3D,0xDD,0xC8,0xB9,0x82,0x50,0x3F,0x8F,0x00,0x4B,0xCA,0x0A,};
const unsigned char sm3_dk_10[32]     = {0xDB,0x72,0x66,0x84,0x5D,0x63,0xB8,0xF4,0xD9,0x31,0xE9,0x0B,0x9F,0xA2,0x77,0x85,
                                   0x60,0x06,0x10,0xDC,0x5B,0xBD,0x06,0xA5,0x5F,0x8B,0xAC,0x79,0x0A,0x83,0xDB,0xE1,};
const unsigned char sm3_dk_11[32]     = {0x35,0x40,0xC8,0x35,0x8E,0x4B,0x1C,0xD6,0x88,0xBB,0x1F,0x9F,0xE3,0xD9,0xFE,0x2D,
                                   0xD7,0xA5,0x09,0xF4,0x1F,0x62,0x76,0x4E,0x8A,0xC5,0x3F,0x7E,0x9A,0x2C,0xD2,0xAD,};

unsigned int pbkdf2_hmac_sm3_test(void)
{
    /******************* inputs from RFC6070 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SM3, password_1, 0, 8, salt_1, 4, 1, sm3_dk_1, 32, "1"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SM3, password_1, 0, 8, salt_1, 4, 2, sm3_dk_2, 32, "2"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SM3, password_1, 0, 8, salt_1, 4, 4096, sm3_dk_3, 32, "3"))
        return 1;

#if 0
    if(pbkdf2_hmac_test_internal(HASH_SM3, password_1, 0, 8, salt_1, 4, 16777216, sm3_dk_4, 32, "4"))
        return 1;
#endif

    if(pbkdf2_hmac_test_internal(HASH_SM3, password_2, 0, 24, salt_2, 36, 4096, sm3_dk_5, 40, "5"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SM3, password_3, 0, 9, salt_3, 5, 4096, sm3_dk_6, 16, "6"))
        return 1;
    /************************************************************/

    /******************* inputs from RFC7914 ********************/
    if(pbkdf2_hmac_test_internal(HASH_SM3, password_4, 0, 6, salt_1, 4, 1, sm3_dk_7, 64, "7"))
        return 1;

    if(pbkdf2_hmac_test_internal(HASH_SM3, password_5, 0, 8, salt_5, 4, 80000, sm3_dk_8, 64, "8"))
        return 1;
    /************************************************************/

    /*********************** corner cases ***********************/
    //password is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SM3, password_1, 0, 0, salt_1, 4, 1, sm3_dk_9, 32, "9"))
        return 1;

    //salt is empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SM3, password_1, 0, 8, salt_1, 0, 2, sm3_dk_10, 32, "10"))
        return 1;

    //password and salt are both empty(length is 0)
    if(pbkdf2_hmac_test_internal(HASH_SM3, password_1, 0, 0, salt_1, 0, 3, sm3_dk_11, 32, "11"))
        return 1;
    /************************************************************/

    printf("\r\n");

    return 0;
}
#endif



unsigned int pbkdf2_speed_test_internal(HASH_ALG hash_alg)
{
    char *name[] = {"SM3","MD5","SHA256","SHA384","SHA512","SHA1","SHA224","SHA512_224","SHA512_256",
            "SHA3_224","SHA3_256","SHA3_384","SHA3_512",};
    unsigned char pwd[16];
    unsigned char salt[16];
    unsigned int pwd_bytes = 16, salt_bytes = 16;
    unsigned char out[16];
    unsigned int out_bytes = 16;
    unsigned int i;
    unsigned int round = 10;
    unsigned int ret;

    uint32_sleep(0xFFFFF, 1);

    printf("\r\n===================== PBDKF2-HMAC %s CPU begin =====================", name[hash_alg]);fflush(stdout);

#ifdef PBKDF2_HMAC_SPEED_TEST_BY_TIMER
    round = 3;
    startP();
#endif

    for(i=0;i<round;i++)
    {
        ret = pbkdf2_hmac(hash_alg, pwd, 0, pwd_bytes, salt, salt_bytes, 5000, out, out_bytes);
        if(HASH_SUCCESS != ret)
        {
            printf("\r\n PBDKF2 error ret=%x", ret); fflush(stdout);
            return 1;
        }
    }

#ifdef PBKDF2_HMAC_SPEED_TEST_BY_TIMER
    endP(1, 0, round);
#else
    printf("\r\n finished");fflush(stdout);
#endif

    return 0;
}

unsigned int pbkdf2_speed_test()
{

#if 1
#ifdef SUPPORT_HASH_SM3
    pbkdf2_speed_test_internal(HASH_SM3);
#endif
#endif

#if 1
#ifdef SUPPORT_HASH_MD5
    pbkdf2_speed_test_internal(HASH_MD5);
#endif
#endif

#if 1
#ifdef SUPPORT_HASH_SHA256
    pbkdf2_speed_test_internal(HASH_SHA256);
#endif
#endif

#if 1
#ifdef SUPPORT_HASH_SHA384
    pbkdf2_speed_test_internal(HASH_SHA384);
#endif
#endif

#if 1
#ifdef SUPPORT_HASH_SHA512
    pbkdf2_speed_test_internal(HASH_SHA512);
#endif
#endif

#if 1
#ifdef SUPPORT_HASH_SHA1
    pbkdf2_speed_test_internal(HASH_SHA1);
#endif
#endif

#if 1
#ifdef SUPPORT_HASH_SHA224
    pbkdf2_speed_test_internal(HASH_SHA224);
#endif
#endif

#if 1
#ifdef SUPPORT_HASH_SHA512_224
    pbkdf2_speed_test_internal(HASH_SHA512_224);
#endif
#endif

#if 1
#ifdef SUPPORT_HASH_SHA512_256
    pbkdf2_speed_test_internal(HASH_SHA512_256);
#endif
#endif

#if 1
#ifdef SUPPORT_HASH_SHA3_224
    //pbkdf2_speed_test_internal(HASH_SHA3_224);
#endif
#endif

#if 1
#ifdef SUPPORT_HASH_SHA3_256
    //pbkdf2_speed_test_internal(HASH_SHA3_256);
#endif
#endif

#if 1
#ifdef SUPPORT_HASH_SHA3_384
    //pbkdf2_speed_test_internal(HASH_SHA3_384);
#endif
#endif

#if 1
#ifdef SUPPORT_HASH_SHA3_512
    //pbkdf2_speed_test_internal(HASH_SHA3_512);
#endif
#endif

    return 0;
}

unsigned int pbkdf2_hmac_test(void)
{
    printf("\r\n\r\n =================== PBKDF2 test =================== ");fflush(stdout);

#if 0
    pbkdf2_speed_test();
#endif

/**/
#ifdef SUPPORT_HASH_MD5
    if(pbkdf2_hmac_md5_test())
        return 1;
#endif

#ifdef SUPPORT_HASH_SHA1
    if(pbkdf2_hmac_sha1_test())
        return 1;
#endif

#ifdef SUPPORT_HASH_SHA224
    if(pbkdf2_hmac_sha224_test())
        return 1;
#endif

#ifdef SUPPORT_HASH_SHA256
    if(pbkdf2_hmac_sha256_test())
        return 1;
#endif

#ifdef SUPPORT_HASH_SHA384
    if(pbkdf2_hmac_sha384_test())
        return 1;
#endif

#ifdef SUPPORT_HASH_SHA512_224
    if(pbkdf2_hmac_sha512_224_test())
        return 1;
#endif

#ifdef SUPPORT_HASH_SHA512_256
    if(pbkdf2_hmac_sha512_256_test())
        return 1;
#endif

#ifdef SUPPORT_HASH_SHA512
    if(pbkdf2_hmac_sha512_test())
        return 1;
#endif

#ifdef SUPPORT_HASH_SM3
    if(pbkdf2_hmac_sm3_test())
        return 1;
#endif
/**/
    printf("\r\n PBKDF2 test success \r\n\r\n");fflush(stdout);

    return 0;
}

void user_init(void)
{
    gpio_function_en(LED1);
    gpio_output_en(LED1);
    gpio_input_dis(LED1);

    pbkdf2_hmac_test();
}

void main_loop(void)
{
    gpio_toggle(LED1);
    delay_ms(50);
}

#endif

